@@ -16,7 +16,7 @@
 
 package MongoDB::Async::Cursor;
 {
-  $MongoDB::Async::Cursor::VERSION = '0.503.2';
+  $MongoDB::Async::Cursor::VERSION = '0.702.3';
 }
 
 
@@ -34,6 +34,8 @@ $MongoDB::Async::Cursor::slave_okay = 0;
 
 $MongoDB::Async::Cursor::timeout = 30000;
 
+# $MongoDB::Async::Cursor::inflate_dbrefs;
+# cache refresher tied here, see BSON.pm
 
 has started_iterating => (
     is => 'rw',
@@ -61,7 +63,6 @@ has _query => (
 
 has _fields => (
     is => 'rw',
-    isa => 'HashRef',
     required => 0,
 );
 
@@ -154,7 +155,9 @@ sub _do_query {
 		$self->{_fields});
     $self->{_request_id} = $info->{'request_id'};
 
-    $self->{_client}->send($query);
+    # $self->{_client}->send($query); # now this in XS too
+	return $query;
+	
 	
     # $self->_client->recv($self);
 	
@@ -176,7 +179,7 @@ sub fields {
     confess "cannot set fields after querying"
 	if $self->{started_iterating};
     confess 'not a hash reference'
-	unless ref $f eq 'HASH';
+		unless ref $f eq 'HASH' || ref $f eq 'Tie::IxHash';
 
     $self->{_fields} = $f;
     return $self;
@@ -190,7 +193,7 @@ sub sort {
 	if $self->{started_iterating};
 	
     confess 'not a hash reference'
-	unless ref $order eq 'HASH' || ref $order eq 'Tie::IxHash';
+		unless ref $order eq 'HASH' || ref $order eq 'Tie::IxHash';
 
 	
     $self->_ensure_special;
@@ -213,7 +216,7 @@ sub limit {
 
 sub tailable {
 	my($self, $bool) = @_;
-	confess "Cannot set tailable state"
+	confess "Cannot set tailable after querying"
 	if $self->{started_iterating};
 	
 	$self->{_tailable} = $bool;
@@ -249,7 +252,7 @@ sub hint {
     confess "cannot set hint after querying"
 	if $self->{started_iterating};
     confess 'not a hash reference'
-	unless ref $index eq 'HASH';
+		unless ref $index eq 'HASH' || ref $index eq 'Tie::IxHash';
 
     $self->_ensure_special;
     $self->{_query}{'$hint'} = $index;
@@ -321,7 +324,7 @@ MongoDB::Async::Cursor - A cursor/iterator for Mongo query results
 
 =head1 VERSION
 
-version 0.503.2
+version 0.702.3
 
 =head1 SYNOPSIS
 
